use byteorder::{ReadBytesExt, WriteBytesExt};
use std::io;
use std::io::{Read, Write};
use crate::{Endian, TryFrom, WithFixedPayloadLength};

#[repr(u16)]
#[derive(Clone, Debug, PartialEq)]
pub enum JobSetInfo {
    ParameterCount = 0x0000,
    JobCount = 0x0001,
    InstructionChunkCount = 0x0002,
    JobState = 0x0003,
    ActiveJobIndex = 0x0004,
}

impl WithFixedPayloadLength for JobSetInfo {
    const FIXED_PAYLOAD_LENGTH: u16 = u16::FIXED_PAYLOAD_LENGTH;
}

impl TryFrom<u16> for JobSetInfo {
    type Err = io::Error;
    fn try_from(value: u16) -> io::Result<Self> {
        match value {
            0x0000 => Ok(JobSetInfo::ParameterCount),
            0x0001 => Ok(JobSetInfo::JobCount),
            0x0002 => Ok(JobSetInfo::InstructionChunkCount),
            0x0003 => Ok(JobSetInfo::JobState),
            0x0004 => Ok(JobSetInfo::ActiveJobIndex),
            _ => Err(io::Error::new(
                io::ErrorKind::InvalidData,
                format!("Invalid JobSetInfo enum value {}", value),
            )),
        }
    }
}

pub(crate) trait JobSetInfoWriterExt: Write {
    fn write_job_set_info(
        &mut self,
        job_set_info: &JobSetInfo,
    ) -> io::Result<()> {
        self.write_u16::<Endian>(job_set_info.clone() as u16)
    }
}

impl<B: Write + ?Sized> JobSetInfoWriterExt for B {}

pub(crate) trait JobSetInfoReaderExt: Read {
    fn read_job_set_info(&mut self) -> io::Result<JobSetInfo> {
        let value = self.read_u16::<Endian>()?;
        JobSetInfo::try_from(value)
    }
}

impl<B: Read + ?Sized> JobSetInfoReaderExt for B {}
